function [] = audiorec(folname,filename)
%UNTITLED Summary of this function goes here
%   Detailed explanation goes here
%stores the values obtained from the gui in 'options'
options = paramSetup;

% timing definitions
sampleRate =44100; %Hz
dataRate = 30.3; %Hz
filter_alpha = .69; %this represents the percentage contribution of the old signal towards the incoming signal. Decreasing this value makes the reticle "more receptive" to sudden changes. 

%sets up nidaq 
if~isempty(findstr('nidaq',options.aidevice_name))
  ai = analoginput('nidaq','Dev1');
  addchannel(ai, options.ai);
   set(ai,'InputType','SingleEnded');
   actualRate = setverify(ai,'SampleRate',sampleRate);
   ai.SamplesPerTrigger = Inf;
   set(ai,'LoggingMode','Memory')
   setverify(ai.Channel,'InputRange',[-5 5]);
end
%sets up windsound
if ~isempty(findstr('winsound',options.aidevice_name))
    ai = analoginput('winsound');
    addchannel(ai, options.ai);
    actualRate = setverify(ai,'SampleRate',sampleRate);
    ai.SamplesPerTrigger = Inf;
else
    return;
end
      
%initializes the calibration figure        
h_fig = figure;
for i=1:length(options.ai)
    subplot(length(options.ai)+1,1,i)
    h_raw(i) = plot(0,0,'b'); hold on;
    h_rms(i) = plot(0,0,'r:');
    h_filt(i) = plot(0,0,'g--');
    h_text(i) = text(0,0,'','HorizontalAlignment','center');
    title(['Channel ',num2str(options.ai(i))])
    legend('Raw data','RMS','Filtered')
end
subplot(length(options.ai)+1,1,length(options.ai)+1)
h_key = plot(0,0);
title('Keypress')
set(gcf,'KeyPressFcn',@(h_obj,evt) ProcessKeyboard(h_obj,evt));
keypress = 0;

all_data = []; 
all_t = [];
draw_t = [];
draw_rms = [];
draw_y = [];
draw_keypress = [];

texthandle = text(0,0,'Press any key to begin','HorizontalAlignment','center');

waiting = 1;
while(waiting)
    drawnow
end

set(texthandle,'String','Collecting- logging keystrokes.  Esc to quit');

tic; % now we start
t = 0;
start(ai)
running = 1;
while isrunning(ai) && running
    % get EMG data (this blocks until it gets the data it wants)
    
    [emg_data,emg_t] = getdata(ai,actualRate/dataRate);
    
    t = toc;
    %concatenate all data
    all_data = [all_data; emg_data];
    all_t = [all_t; emg_t];

    draw_t = [draw_t; t];
    draw_keypress = [draw_keypress; keypress]; keypress = 0; % handle any keypress
    if ~exist('draw_rms','var')
        draw_rms = [];
    end
    draw_rms = [draw_rms; zeros(1,size(emg_data,2))];
    for i=1:length(options.ai)
        draw_rms(end,i) = sqrt(mean(emg_data(:,i).^2)); %computes the rms for each chunk of incoming data
    end
    if ~isempty(draw_y)
        draw_y = [draw_y; filter_alpha*draw_y(end,:) + (1-filter_alpha)*(draw_rms(end,:))]; %computes filtered signal
    else
        draw_y = draw_rms(end,:);
    end
end
stop(ai)

delete(texthandle);
%plots the collected/calculated data above
for i=1:length(options.ai)
    set(h_raw(i),'XData',all_t,'YData',all_data(:,i))
    set(h_rms(i),'XData',draw_t,'YData',draw_rms(:,i))
    set(h_filt(i),'XData',draw_t,'YData',draw_y(:,i))    
    set(h_text(i),'Position',[mean(all_t) mean(all_data(:,i)) 0],...
        'HorizontalAlignment','center',...
        'String',['Max RMS= ' num2str(max(draw_rms(:,i))) '  Max Filtered=',num2str(max(draw_y(:,i)))])
    fprintf(['\nChannel ',num2str(options.ai(i)) '\nMax RMS= ' num2str(max(draw_rms(:,i))) '\nMax Filtered=' num2str(max(draw_y(:,i))) '\nMin Filtered=' num2str(min(draw_y(:,i))) ])
end

%plots keypress
set(h_key,'XData',draw_t,'YData',draw_keypress);
drawnow;

%saves a file containting the variables listed
save([folname filename '.mat'],'all_t','all_data','draw_t','draw_rms','draw_y','draw_keypress');

%save wavefile
audio_y = all_data;
fs = 1/(all_t(2) - all_t(1));
wavwrite(audio_y,fs,16,[folname filename]);

%creates the keypress function used above
function [] = ProcessKeyboard(foo,evt)
keypress = 1;
if strmatch(evt.Key,'escape','exact')
    running = 0;
end
if (waiting)
    waiting = 0 ;
end
end


end

