function [] = find_RFFcycles(audiofolname,filename)
clear aRFF; close all;
global perform_fol; 

%declaring subject folders
aRFF.subjectid = filename(1:end-4);
folname = [perform_fol aRFF.subjectid '\'];
varfolname = [folname 'var\'];
fricfile = [varfolname 'fric_param.mat'];

%if we do not have the fricfile, need to go back and create it
if ~exist(fricfile,'file')
    find_fric(folname, filename)
    return;
else
    load(fricfile);
end

%plot audio waveform and processed signals
figure('units','normalized','outerposition',[0 0 1 1]); hold on;
subplot(4,1,1)
plot(aRFF.t,aRFF.y)
xlabel('Time (s)')
ylabel('Audio Waveform (X)')

subplot(4,1,2); hold all;
plot(aRFF.t,v_prob')
plot(aRFF.t,f_prob')
diff_prob = smooth(f_prob - v_prob,round(aRFF.fs/20));
plot(aRFF.t,diff_prob)
legend('X_{VowelLike}','X_{FricativeLike}','Difference')
ylabel('Filtered Signal')
xlabel('Time (s)')
hold all;

subplot(4,1,3); hold all;
plot(aRFF.t,aRFF.f0y)
plot(aRFF.t(pk_loc),aRFF.f0y(pk_loc),'.','MarkerSize',25)
xlabel('Time (s)')
ylabel('Filtered Waveform X_{F0}')
legend('X_{F0}','L_{Fricatives}')

%Initialize Threshvect. This is currently tuned to manual RFF estimates.
offshiftthresh = -0.05; %-0.10 change thresh by 10%, + gets closer to VC, - gets further from VC
onshiftthresh = 0.10; %0.10 change thresh by 10%

%for each RFF instance, calculate offset RFF, then calculate onset RFF. Save data into performance folder.
for instancei = 1:totalinst
    aRFF.instancei = instancei;
    fric_center = pk_loc(aRFF.instancei);
    
    %%for calculating offset RFF
    aRFF.onoff = 'off';
    instancefile = [varfolname num2str(instancei) '_' aRFF.onoff '.mat'];
    
    %compute average period
    off_t0_step = round(aRFF.fs/offpitch(instancei));
    
    %first pulse located at 3Tavg + fricative location. Next pulse is
    %Tavg before.
    Tendbackpulse = fric_center + 3*off_t0_step;
    Pendbackpulse = fric_center + 3*off_t0_step;
    Tbackpulse = Tendbackpulse - off_t0_step;
    Pbackpulse = Pendbackpulse - off_t0_step;
    
    aRFF.peakcyclefull = 0; %1 if we have at least 10 peak pulses
    aRFF.troughcyclefull = 0; %1 if we have at least 10 trough pulses
    aRFF.trough_array = []; %array of trough cycle
    aRFF.peak_array = []; %array of peak cycle
    
    aRFF.PulseFindingDone = 0; %1 if quit the while loop
    aRFF.checkedonce = 0; %ran through checking_cycles at least once
    donewithdeletion = 0;
    aRFF.recordingedge = offedge(aRFF.instancei);
    
    trough_zc_all = []; %store zc
    peak_zc_all = []; %store zc
    trough_ptpratio = []; %store ptp
    peak_ptpratio = []; %store ptp
    last_trough_vect = []; %this is the trough vect for the last cycle. used to calc ss
    last_peak_vect = []; %this is the peak vect for teh last cycle. used to calc ss
    trough_ss_all = []; %store ss
    peak_ss_all = []; %store ss
    
    while ~aRFF.PulseFindingDone
        if min(Tbackpulse, Pbackpulse)-1< aRFF.recordingedge
            plot_pulses(aRFF)
            break;
        end
        
        %finding cycle trough
        trough_l = trough_find(aRFF, Tbackpulse, Tendbackpulse);
        
        %finding cycle peak
        peak_l = peak_find(aRFF, Pbackpulse, Pendbackpulse);
        
        %if we've reached the start of the offset vowel, plot and quit
        if min(trough_l, peak_l)-off_t0_step < aRFF.recordingedge
            plot_pulses(aRFF)
            break;
        end
        
        cycle_trough_l = trough_l-off_t0_step:trough_l;
        cycle_peak_l = peak_l-off_t0_step:peak_l;
        
        %compute parameters
        troughptp = range(aRFF.y(cycle_trough_l)); %peak-to-peak amplitude
        troughmean = mean(aRFF.y(cycle_trough_l)); %mean
        trough_zc = zero_crossings(aRFF.y(cycle_trough_l)-troughmean); %zero crossings
        
        if isempty(last_trough_vect) %shape similarity
            trough_ss = 1;
        else
            ptspace = linspace(1,length(cycle_trough_l),length(last_trough_vect));
            vq = interp1(1:length(cycle_trough_l),aRFF.y(cycle_trough_l),ptspace);
            trough_ss = procrustes(last_trough_vect,vq','reflection',false);
        end
        
        peakptp = range(aRFF.y(cycle_peak_l)); %peak-to-peak amplitude
        peakmean = mean(aRFF.y(cycle_peak_l)); %mean
        peak_zc = zero_crossings(aRFF.y(cycle_peak_l)-peakmean); %zero crossings
        
        if isempty(last_peak_vect) %shape similarity
            peak_ss = 1;
        else
            ptspace = linspace(1,length(cycle_peak_l),length(last_peak_vect));
            vq = interp1(1:length(cycle_peak_l),aRFF.y(cycle_peak_l),ptspace);
            peak_ss = procrustes(last_peak_vect,vq','reflection',false);
        end
        
        %storing varables for new troughs
        if ~ismember(trough_l,aRFF.trough_array)
            aRFF.trough_array = [aRFF.trough_array trough_l];
            trough_zc_all = [trough_zc_all trough_zc];
            trough_ptpratio = [trough_ptpratio troughptp];
            trough_ss_all = [trough_ss_all trough_ss];
            
            last_trough_vect = aRFF.y(cycle_trough_l);
        end
        
        %storing varables for new peaks
        if ~ismember(peak_l,aRFF.peak_array)
            aRFF.peak_array = [aRFF.peak_array peak_l];
            peak_zc_all = [peak_zc_all peak_zc];
            peak_ptpratio = [peak_ptpratio peakptp];
            peak_ss_all = [peak_ss_all peak_ss];
            
            last_peak_vect = aRFF.y(cycle_peak_l);
        end
        
        %maximum period allowed is 1.5*Tavg
        max_acf_range = floor(off_t0_step*1.5);
        
        %if there are at least five pulses and the last six cycles
        %have similar shapes, the try to determine first peak
        if ~donewithdeletion & (length(trough_zc_all)>5) & (all(trough_ss_all(end-4:end)<0.8)) 
                    %%if ~donewithdeletion & (length(trough_zc_all)>5) &
                    %%(all(trough_ss_all(end-4:end)<0.5)) DEFAULT
            long_enough = (aRFF.trough_array(end-4) - aRFF.trough_array(end) + 1) > max_acf_range;
            if ~long_enough
                Tendbackpulse = peak_l;
                Tbackpulse = peak_l - off_t0_step;
                Pendbackpulse = trough_l;
                Pbackpulse = trough_l - off_t0_step;
                continue;
            end
            acf = autocorr(aRFF.y(aRFF.trough_array(end):aRFF.trough_array(end-4)),max_acf_range);
            maxacf = max(acf(floor(off_t0_step*0.5):max_acf_range));
            if (maxacf>0.4) | (sum(trough_ss_all < 0.8) > 10)
            %%if (maxacf>0.7) & ((~diff_prob(Tbackpulse) >
            %%diff_prob(Tendbackpulse)) | (sum(trough_ss_all < 0.5) > 10))
            %%DEFAULT
                trough_zc_thresh = (1+offshiftthresh)*threshfind(trough_zc_all);
                trough_ptp_thresh = (1-offshiftthresh)*threshfind(trough_ptpratio);
                trough_ss_thresh = (1+offshiftthresh)*threshfind(trough_ss_all);
                peak_zc_thresh = (1+offshiftthresh)*threshfind(peak_zc_all);
                peak_ptp_thresh = (1-offshiftthresh)*threshfind(peak_ptpratio);
                peak_ss_thresh = (1+offshiftthresh)*threshfind(peak_ss_all);
                
                deltroughzc = find(trough_zc_all>trough_zc_thresh,1,'last');
                deltroughptp = find(trough_ptpratio<trough_ptp_thresh,1,'last');
                deltroughss = find(trough_ss_all>trough_ss_thresh,1,'last');
                deltrough = round(median([deltroughzc deltroughptp deltroughss]));
                [deltroughmax,~] = max([deltroughzc deltroughptp deltroughss]);
                
                %if the autocorrelation of the waveform between the mid
                %threshold and the max threshold is less than 0.3, mid
                %we want to delete up to the max threshold
                long_enough = (aRFF.trough_array(deltrough) - aRFF.trough_array(deltroughmax) + 1) > max_acf_range;
                if deltroughmax - deltrough > 3 & long_enough
                    acf = autocorr(aRFF.y(aRFF.trough_array(deltroughmax):aRFF.trough_array(deltrough)),max_acf_range);
                    maxacf = max(acf(floor(off_t0_step*0.5):max_acf_range));
                    if maxacf < 0.3
                        deltrough = deltroughmax;
                    end
                end
                
                delpeakzc = find(peak_zc_all>peak_zc_thresh,1,'last');
                delpeakptp = find(peak_ptpratio<peak_ptp_thresh,1,'last');
                delpeakss = find(peak_ss_all>peak_ss_thresh,1,'last');
                delpeak = round(median([delpeakzc delpeakptp delpeakss]));
                [delpeakmax,~] = max([delpeakzc delpeakptp delpeakss]);
                
                %if the autocorrelation of the waveform between the mid
                %threshold and the max threshold is less than 0.3, mid
                %we want to delete up to the max threshold
                long_enough = (aRFF.peak_array(delpeak) - aRFF.peak_array(delpeakmax) + 1) > max_acf_range;
               if  delpeakmax - delpeak > 3 & long_enough 
                     acf = autocorr(aRFF.y(aRFF.peak_array(delpeakmax):aRFF.peak_array(delpeak)),max_acf_range);
                    maxacf = max(acf(floor(off_t0_step*0.5):max_acf_range));
                    if maxacf < 0.15 
                       delpeak = delpeakmax;
                    end
                end
                
                %if there is nothing to delete, then get more pulses
                if isnan(deltrough) | isnan(delpeak)
                    Tendbackpulse = peak_l;
                    Tbackpulse = peak_l - off_t0_step;
                    Pendbackpulse = trough_l;
                    Pbackpulse = trough_l - off_t0_step;
                    continue;
                end
                
                if min(trough_ss_all) > 0.5
                    deltrough = length(trough_ss_all);
                end
                if min(peak_ss_all) > 0.5
                    delpeak = length(peak_ss_all);
                end
                
                %troughtemp and peaktemp should only have pulses where
                %there are cycles
                troughtemp = aRFF.trough_array;
                peaktemp = aRFF.peak_array;
                troughtemp(1:deltrough) = [];
                peaktemp(1:delpeak) = [];
                
                %re-estimate F0
                [off_t0_step,aRFF] = finer_t0_est(Tbackpulse,aRFF,off_t0_step);
                
                %troughtemp and peaktemp is long enough, then actually
                %delete the cycles
                if (length(troughtemp) > 4) & (length(peaktemp) > 4)
                    donewithdeletion = 1;
                    aRFF.trough_array(1:deltrough) = [];
                    aRFF.peak_array(1:delpeak) = [];
                end
            end
        end
        %indicates that there are at least 11 trough cycle pulses
        if length(aRFF.trough_array)>10 & donewithdeletion
            aRFF.troughcyclefull = 1;
        end
        %indicates that there are at least 11 peak cycle pulses
        if length(aRFF.peak_array)>10 & donewithdeletion
            aRFF.peakcyclefull = 1;
        end
        
        if aRFF.peakcyclefull & aRFF.troughcyclefull
            aRFF = checking_cycles(aRFF,off_t0_step); %check cycles
            if isempty(aRFF.trough_array)
                break;
            end
            if length(aRFF.trough_array) < 11 & length(aRFF.peak_array) < 11
                break;
            end
            aRFF = calcRFF(aRFF); % calculate RFF
            plot_pulses(aRFF); % plot cycle pulses
            
            subplot(4,1,4); hold off; % plot RFF
            if isfield(aRFF,'trough_RFF_Hz')
                if ~isnan(aRFF.trough_RFF_Hz)
                    plot(1:10, aRFF.trough_RFF(end-9:end),'.-')
                else
                    plot(1:10, aRFF.peak_RFF(end-9:end),'.-')
                end
            end
            
            h = figure(1) %save figure
            saveas(h,[folname num2str(instancei) '_' aRFF.onoff '.jpg'])
            aRFF.PulseFindingDone = 1;
        end
        
        Tendbackpulse = peak_l;
        Tbackpulse = peak_l - off_t0_step;
        
        Pendbackpulse = trough_l;
        Pbackpulse = trough_l - off_t0_step;
    end
    save(instancefile,'aRFF') % save RFF
    
    %clear variables.
    if isfield(aRFF,'trough_RFF_Hz')
        delfields = {'trough_RFF_Hz','peak_RFF_Hz','trough_RFF','peak_RFF','trough_RFF_meanf0','peak_RFF_meanf0'};
        aRFF = rmfield(aRFF,delfields);
    end
    
    %%for calculating onset RFF
    aRFF.onoff = 'on';
    instancefile = [varfolname num2str(instancei) '_' aRFF.onoff '.mat'];
    
    on_t0_step = round(aRFF.fs/onpitch(instancei));
    Tstartpulse = fric_center - 3*on_t0_step;
    Pstartpulse = fric_center - 3*on_t0_step;
    Tfrontpulse = Tstartpulse + on_t0_step;
    Pfrontpulse = Pstartpulse + on_t0_step;
    
    aRFF.peakcyclefull = 0;
    aRFF.troughcyclefull = 0;
    aRFF.trough_array = [];
    aRFF.peak_array = [];
    aRFF.PulseFindingDone = 0; %quit the while loop
    aRFF.checkedonce = 0;
    aRFF.recordingedge = onedge(aRFF.instancei);
    
    trough_zc_all = [];
    peak_zc_all = [];
    trough_ptpratio = [];
    peak_ptpratio = [];
    last_trough_vect = []; %calc ss
    last_peak_vect = []; %calc ss
    trough_ss_all = []; %calc ss
    peak_ss_all = []; %calc ss
    donewithdeletion = 0;
    
    while ~aRFF.PulseFindingDone
        if max(Tfrontpulse, Pfrontpulse)+1 > aRFF.recordingedge
            break;
        end
        
        %finding cycle trough
        trough_l = trough_find(aRFF, Tstartpulse, Tfrontpulse);
        
        %finding cycle peak
        peak_l = peak_find(aRFF, Pstartpulse, Pfrontpulse);
        
        if max(peak_l, trough_l)+on_t0_step > aRFF.recordingedge
            break;
        end
        
        cycle_trough_l = trough_l:trough_l+on_t0_step;
        cycle_peak_l = peak_l:peak_l+on_t0_step;
        
        troughptp = range(aRFF.y(cycle_trough_l));
        troughcenter = mean(aRFF.y(cycle_trough_l));
        trough_zc = zero_crossings(aRFF.y(cycle_trough_l)-troughcenter);
        
        if isempty(last_trough_vect)
            trough_ss = 1;
        else
            ptspace = linspace(1,length(cycle_trough_l),length(last_trough_vect));
            vq = interp1(1:length(cycle_trough_l),aRFF.y(cycle_trough_l),ptspace);
            trough_ss = procrustes(last_trough_vect,vq','reflection',false);
        end
        
        peakptp = range(aRFF.y(cycle_peak_l));
        peakcenter = mean(aRFF.y(cycle_peak_l));
        peak_zc = zero_crossings(aRFF.y(cycle_peak_l)-peakcenter);
        if isempty(last_peak_vect)
            peak_ss = 1;
        else
            ptspace = linspace(1,length(cycle_peak_l),length(last_peak_vect));
            vq = interp1(1:length(cycle_peak_l),aRFF.y(cycle_peak_l),ptspace);
            peak_ss = procrustes(last_peak_vect,vq','reflection',false);
        end
        
        if ~ismember(trough_l,aRFF.trough_array)
            aRFF.trough_array = [aRFF.trough_array trough_l];
            trough_zc_all = [trough_zc_all trough_zc];
            trough_ptpratio = [trough_ptpratio troughptp];
            trough_ss_all = [trough_ss_all trough_ss];

            last_trough_vect = aRFF.y(cycle_trough_l);
        end
        if ~ismember(peak_l,aRFF.peak_array)
            aRFF.peak_array = [aRFF.peak_array peak_l];
            peak_zc_all = [peak_zc_all peak_zc];
            peak_ptpratio = [peak_ptpratio peakptp];
            peak_ss_all = [peak_ss_all peak_ss];
            
            last_peak_vect = aRFF.y(cycle_peak_l);
        end
        
        max_acf_range = floor(on_t0_step*1.5);
        
        if ~donewithdeletion & (length(trough_zc_all)>5) & (all(trough_ss_all(end-4:end)<0.8))
            long_enough = (aRFF.trough_array(end) - aRFF.trough_array(end-4) + 1) > max_acf_range;
            if ~long_enough
                Tstartpulse = peak_l;
                Tfrontpulse = peak_l + on_t0_step;
                Pstartpulse = trough_l;
                Pfrontpulse = trough_l + on_t0_step;
                continue;
            end
            acf = autocorr(aRFF.y(aRFF.trough_array(end-4):aRFF.trough_array(end)),max_acf_range);
            maxacf = max(acf(floor(on_t0_step*0.5):max_acf_range));
            if (maxacf > 0.4) | (sum(trough_ss_all < 0.8) > 10)
                trough_zc_thresh = (1+onshiftthresh)*threshfind(trough_zc_all);
                trough_ptp_thresh = (1-onshiftthresh)*threshfind(trough_ptpratio);
                trough_ss_thresh = (1+onshiftthresh)*threshfind(trough_ss_all);
                peak_zc_thresh = (1+onshiftthresh)*threshfind(peak_zc_all);
                peak_ptp_thresh = (1-onshiftthresh)*threshfind(peak_ptpratio);
                peak_ss_thresh = (1+onshiftthresh)*threshfind(peak_ss_all);
                
                deltroughzc = find(trough_zc_all>trough_zc_thresh,1,'last');
                deltroughptp = find(trough_ptpratio<trough_ptp_thresh,1,'last');
                deltroughss = find(trough_ss_all>trough_ss_thresh,1,'last');
                deltrough = round(median([deltroughzc deltroughptp deltroughss]));
                [deltroughmax,~] = max([deltroughzc deltroughptp deltroughss]);
                
                long_enough = (aRFF.trough_array(deltroughmax) - aRFF.trough_array(deltrough) + 1) > max_acf_range;
                if deltroughmax - deltrough > 3 & long_enough
                    acf = autocorr(aRFF.y(aRFF.trough_array(deltrough):aRFF.trough_array(deltroughmax)),max_acf_range);
                    maxacf = max(acf(floor(on_t0_step*0.5):floor(on_t0_step*1.5)));
                    if maxacf < 0.15
                        deltrough = deltroughmax;
                    end
                end
                
                delpeakzc = find(peak_zc_all>peak_zc_thresh,1,'last');
                delpeakptp = find(peak_ptpratio<peak_ptp_thresh,1,'last');
                delpeakss = find(peak_ss_all>peak_ss_thresh,1,'last');
                delpeak = round(median([delpeakzc delpeakptp delpeakss]));
                [delpeakmax,~] = max([delpeakzc delpeakptp delpeakss]);
                
                long_enough = (aRFF.peak_array(delpeakmax) - aRFF.peak_array(delpeak) + 1) > max_acf_range;
                if delpeakmax - delpeak > 3 & long_enough
                    acf = autocorr(aRFF.y(aRFF.peak_array(delpeak):aRFF.peak_array(delpeakmax)),max_acf_range);
                    maxacf = max(acf(floor(on_t0_step*0.5):floor(on_t0_step*1.5)));
                    if maxacf < 3
                        delpeak = delpeakmax;
                    end
                end
                
                if isnan(deltrough) | isnan(delpeak)
                    Tstartpulse = peak_l;
                    Tfrontpulse = peak_l + on_t0_step;
                    Pstartpulse = trough_l;
                    Pfrontpulse = trough_l + on_t0_step;
                    continue;
                end
                
                if min(trough_ss_all) > 0.5
                    deltrough = length(trough_ss_all);
                end
                if min(peak_ss_all) > 0.5
                    delpeak = length(peak_ss_all);
                end
                
                troughtemp = aRFF.trough_array;
                peaktemp = aRFF.peak_array;
                troughtemp(1:deltrough) = [];
                peaktemp(1:delpeak) = [];
                
                [on_t0_step,aRFF] = finer_t0_est(Tfrontpulse,aRFF,on_t0_step);
                
                if (length(troughtemp) > 4) & (length(peaktemp) > 4)
                    aRFF.trough_array(1:deltrough) = [];
                    aRFF.peak_array(1:delpeak) = [];
                    donewithdeletion = 1;
                end
            end
        end
        
        if length(aRFF.trough_array)>10 & donewithdeletion
            aRFF.troughcyclefull = 1;
        end
        
        if length(aRFF.peak_array)>10 & donewithdeletion
            aRFF.peakcyclefull = 1;
        end
        
        if aRFF.peakcyclefull & aRFF.troughcyclefull
            aRFF = checking_cycles(aRFF,on_t0_step);
            if isempty(aRFF.trough_array)
                break;
            end
            if length(aRFF.trough_array) < 11 & length(aRFF.peak_array) < 11
                break;
            end
            aRFF = calcRFF(aRFF);
            plot_pulses(aRFF);
            
            subplot(4,1,4); hold on;
            if isfield(aRFF,'trough_RFF_Hz')
                if ~isnan(aRFF.trough_RFF_Hz)
                    plot(14:23, aRFF.trough_RFF(1:10),'.-')
                else
                    plot(14:23, aRFF.peak_RFF(1:10),'.-')
                end
                xlim([1 24])
            end
            h = figure(1)
            
            saveas(h,[folname num2str(instancei) '_' aRFF.onoff '.jpg'])
            aRFF.PulseFindingDone = 1;
        end
        
        Tstartpulse = peak_l;
        Tfrontpulse = peak_l + on_t0_step;
        Pstartpulse = trough_l;
        Pfrontpulse = trough_l + on_t0_step;
    end
    save(instancefile,'aRFF')
    
    if isfield(aRFF,'trough_RFF_Hz')
        delfields = {'trough_RFF_Hz','peak_RFF_Hz','trough_RFF','peak_RFF','trough_RFF_meanf0','peak_RFF_meanf0'};
        aRFF = rmfield(aRFF,delfields);
    end
end

close all;

